import { SimpleCache, WithSimpleCachingOnSetTrigger } from '.';
import { WithSimpleCachingOptions } from './withSimpleCaching';
/**
 * a simple typeguard which checks if an object has a property named `forInput`
 */
export declare const hasForInputProperty: (obj: any) => obj is {
    forInput: any;
};
/**
 * the shape of logic that was wrapped with extendable caching
 */
export interface LogicWithExtendableCaching<LR extends any, CR extends any, L extends (...args: any[]) => LR> {
    /**
     * execute the logic with caching
     */
    execute: L;
    /**
     * invalidate the cached value for a given input
     *
     * note
     * - applies key serialization on the key input, just like execute
     */
    invalidate: (args: {
        /**
         * invalidate the cache for this input
         */
        forInput: Parameters<L>;
    } | {
        /**
         * invalidate the cache for this key
         */
        forKey: string;
        /**
         * the cache to use, if the cache must be was defined from input parameters at runtime
         */
        cache?: SimpleCache<CR>;
    }) => Promise<void>;
    /**
     * update the cached value for a given input
     *
     * note
     * - applies key serialization on the key input, just like execute
     * - applies value serialization on the value output, just like execute
     */
    update: (args: {
        /**
         * update the cache for this input
         */
        forInput: Parameters<L>;
        /**
         * update the cache to this value
         */
        toValue: ReturnType<L> | ((args: {
            cachedValue: CR | undefined;
        }) => ReturnType<L>);
    } | {
        /**
         * update the cache for this string
         */
        forKey: string;
        /**
         * update the cache to this value
         */
        toValue: ReturnType<L> | ((args: {
            cachedValue: CR | undefined;
        }) => ReturnType<L>);
        /**
         * the cache to use, if the cache must be was defined from input parameters at runtime
         */
        cache?: SimpleCache<CR>;
    }) => Promise<void>;
}
/**
 * a function which is capable of grabbing the cache from arguments to the `invalidate` or `update` commands, supporting both the case when invoked with `forInput` and when invoked with `forKey`
 */
export declare const getCacheFromCacheOptionOrFromForKeyArgs: <LR extends unknown, CR extends unknown, L extends (...args: any[]) => LR>({ args, options, trigger, }: {
    args: {
        /**
         * invalidate the cache for this input
         */
        forInput: Parameters<L>;
    } | {
        /**
         * invalidate the cache for this key
         */
        forKey: string;
        /**
         * the cache to use, if the cache must be was defined from input parameters at runtime
         */
        cache?: SimpleCache<CR> | undefined;
    } | {
        /**
         * update the cache for this input
         */
        forInput: Parameters<L>;
        /**
         * update the cache to this value
         */
        toValue: ReturnType<L> | ((args: {
            cachedValue: CR | undefined;
        }) => ReturnType<L>);
    } | {
        /**
         * update the cache for this string
         */
        forKey: string;
        /**
         * update the cache to this value
         */
        toValue: ReturnType<L> | ((args: {
            cachedValue: CR | undefined;
        }) => ReturnType<L>);
        /**
         * the cache to use, if the cache must be was defined from input parameters at runtime
         */
        cache?: SimpleCache<CR> | undefined;
    };
    options: WithSimpleCachingOptions<LR, CR, L>;
    trigger: WithSimpleCachingOnSetTrigger;
}) => SimpleCache<CR>;
/**
 * exposes the cache-wrapped method along with some primitives which enable extending the caching logic
 *
 * specifically
 * - exposes a way to `invalidate` the cache, for a given input (e.g., to support external triggers for invalidation)
 * - exposes a way to `update` the cache, for a given input (e.g., to support write-through caching and optimistic caching)
 *
 * relevance
 * - when wrapping logic to cache the user is able to specify several caching options (e.g., key serialization method, value serialization method, etc)
 * - in order to define their own `invalidation` and `update` methods, without this function, the user would need to access these caching options per function elsewhere
 * - this function makes it easy to utilize and extend cache invalidation + update commands for the wrapped logic, by managing the references to the caching options on behalf of the user
 */
export declare const withExtendableCaching: <LR extends unknown, CR extends unknown, L extends (...args: any[]) => LR>(logic: L, options: WithSimpleCachingOptions<LR, CR, L>) => LogicWithExtendableCaching<LR, CR, L>;
