"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.withExtendableCaching = exports.getCacheFromCacheOptionOrFromForKeyArgs = exports.hasForInputProperty = void 0;
const _1 = require(".");
const BadRequestError_1 = require("./errors/BadRequestError");
const isAFunction_1 = require("./isAFunction");
const withSimpleCaching_1 = require("./withSimpleCaching");
/**
 * a simple typeguard which checks if an object has a property named `forInput`
 */
const hasForInputProperty = (obj) => !!obj.forInput;
exports.hasForInputProperty = hasForInputProperty;
/**
 * a function which is capable of grabbing the cache from arguments to the `invalidate` or `update` commands, supporting both the case when invoked with `forInput` and when invoked with `forKey`
 */
const getCacheFromCacheOptionOrFromForKeyArgs = ({ args, options, trigger, }) => {
    // if the args have the forInput property, then we can grab the cache like normal
    if ((0, exports.hasForInputProperty)(args))
        return (0, withSimpleCaching_1.getCacheFromCacheOption)({ forInput: args.forInput, cacheOption: options.cache });
    // otherwise, if the cache was explicitly declared, then use it
    if (!(0, isAFunction_1.isAFunction)(options.cache))
        return options.cache;
    // otherwise, we require the cache to have ben defined as an input to this method, when using invalidate by input, since we expect to grab the cache off of the input
    if (!args.cache)
        throw new BadRequestError_1.BadRequestError(`could not find the cache to ${trigger.toLowerCase()} in. ${trigger.toLowerCase()} was called forKey but the cache for this method was defined as a function which retrieves the cache from the input. therefore, since there is no input accessible, the cache should have been explicitly passed in on ${trigger.toLowerCase()}`, { args });
    return args.cache;
};
exports.getCacheFromCacheOptionOrFromForKeyArgs = getCacheFromCacheOptionOrFromForKeyArgs;
/**
 * exposes the cache-wrapped method along with some primitives which enable extending the caching logic
 *
 * specifically
 * - exposes a way to `invalidate` the cache, for a given input (e.g., to support external triggers for invalidation)
 * - exposes a way to `update` the cache, for a given input (e.g., to support write-through caching and optimistic caching)
 *
 * relevance
 * - when wrapping logic to cache the user is able to specify several caching options (e.g., key serialization method, value serialization method, etc)
 * - in order to define their own `invalidation` and `update` methods, without this function, the user would need to access these caching options per function elsewhere
 * - this function makes it easy to utilize and extend cache invalidation + update commands for the wrapped logic, by managing the references to the caching options on behalf of the user
 */
const withExtendableCaching = (logic, options) => {
    const execute = (0, _1.withSimpleCaching)(logic, options);
    const invalidate = (args) => __awaiter(void 0, void 0, void 0, function* () {
        var _a, _b, _c;
        // define how to get the cache, with support for `forKey` input instead of full input
        const cache = (0, exports.getCacheFromCacheOptionOrFromForKeyArgs)({ args, options, trigger: _1.WithSimpleCachingOnSetTrigger.INVALIDATE });
        // define the key, with support for `forKey` input instead of `forInput`
        const serializeKey = (_b = (_a = options.serialize) === null || _a === void 0 ? void 0 : _a.key) !== null && _b !== void 0 ? _b : withSimpleCaching_1.defaultKeySerializationMethod;
        const key = (0, exports.hasForInputProperty)(args) ? serializeKey({ forInput: args.forInput }) : args.forKey;
        // set undefined into the cache for this key, to invalidate the cached value
        yield cache.set(key, undefined);
        // trigger the onSet hook, if needed
        if ((_c = options.hook) === null || _c === void 0 ? void 0 : _c.onSet)
            // note: we do not wait for the hook to resolve; hooks do not block execution ℹ️
            options.hook.onSet({
                trigger: _1.WithSimpleCachingOnSetTrigger.INVALIDATE,
                forInput: (0, exports.hasForInputProperty)(args) ? args.forInput : undefined,
                forKey: key,
                value: undefined,
            });
    });
    const update = (args) => __awaiter(void 0, void 0, void 0, function* () {
        var _d, _e, _f, _g, _h;
        // define how to get the cache, with support for `forKey` input instead of full input
        const cache = (0, exports.getCacheFromCacheOptionOrFromForKeyArgs)({ args, options, trigger: _1.WithSimpleCachingOnSetTrigger.UPDATE });
        // define the key, with support for `forKey` input instead of `forInput`
        const serializeKey = (_e = (_d = options.serialize) === null || _d === void 0 ? void 0 : _d.key) !== null && _e !== void 0 ? _e : withSimpleCaching_1.defaultKeySerializationMethod;
        const key = (0, exports.hasForInputProperty)(args) ? serializeKey({ forInput: args.forInput }) : args.forKey;
        // define the serialized value
        const serializeValue = (_g = (_f = options.serialize) === null || _f === void 0 ? void 0 : _f.value) !== null && _g !== void 0 ? _g : withSimpleCaching_1.defaultValueSerializationMethod;
        const newValue = (0, isAFunction_1.isAFunction)(args.toValue) ? args.toValue({ cachedValue: yield cache.get(key) }) : args.toValue;
        const serializedValue = serializeValue(newValue);
        // set the new value for this key
        yield cache.set(key, serializedValue, { secondsUntilExpiration: options.secondsUntilExpiration });
        // trigger the onSet hook, if needed
        if ((_h = options.hook) === null || _h === void 0 ? void 0 : _h.onSet)
            // note: we do not wait for the hook to resolve; hooks do not block execution ℹ️
            options.hook.onSet({
                trigger: _1.WithSimpleCachingOnSetTrigger.UPDATE,
                forInput: (0, exports.hasForInputProperty)(args) ? args.forInput : undefined,
                forKey: key,
                value: {
                    output: newValue,
                    cached: serializedValue,
                },
            });
    });
    return { execute, invalidate, update };
};
exports.withExtendableCaching = withExtendableCaching;
//# sourceMappingURL=withExtendableCaching.js.map