export interface SimpleCache<T> {
    get: (key: string) => T | undefined | Promise<Awaited<T> | undefined>;
    set: (key: string, value: T, options?: {
        secondsUntilExpiration?: number;
    }) => void | Promise<void>;
}
export declare type KeySerializationMethod<P> = (args: {
    forInput: P;
}) => string;
export declare const noOp: <LR, CR>(value: LR) => CR;
export declare const defaultKeySerializationMethod: KeySerializationMethod<any>;
export declare const defaultValueSerializationMethod: <LR, CR>(value: LR) => CR;
export declare type SimpleCacheResolutionMethod<L extends (...args: any[]) => any, CR extends any> = (args: {
    fromInput: Parameters<L>;
}) => SimpleCache<CR>;
export declare const getCacheFromCacheOption: <L extends (...args: any[]) => any, CR extends unknown>({ forInput, cacheOption, }: {
    forInput: Parameters<L>;
    cacheOption: SimpleCache<CR> | SimpleCacheResolutionMethod<L, CR>;
}) => SimpleCache<CR>;
/**
 * enumerates all of the methods exposed by with-simple-caching which are capable of setting to the cache
 */
export declare enum WithSimpleCachingOnSetTrigger {
    /**
     * the execute method sets to the cache when your wrapped logic is executed
     */
    EXECUTE = "EXECUTE",
    /**
     * the invalidate method sets to the cache when the invalidate method is executed
     */
    INVALIDATE = "INVALIDATE",
    /**
     * the update method sets to the cache when the update method is executed
     */
    UPDATE = "UPDATE"
}
/**
 * a hook which is triggered onSet to the cache
 *
 * note
 * - this hook is simply kicked off, it will not be awaited, therefore
 *   - make sure you catch any errors thrown any promises you may start in the hook
 * - this hook exposes the exact value that was used when setting to cache
 *   - it exposes the exact output value, returned by the logic (if logic is async, this will be a promise)
 *   - it exposes the exact serialized value, given to cache onSet (if logic was async, this will be a promise)
 */
export declare type SimpleCacheOnSetHook<LR extends any, CR extends any, L extends (...args: any[]) => LR> = (args: {
    /**
     * the method which triggered this onSet hook
     */
    trigger: WithSimpleCachingOnSetTrigger;
    /**
     * the input for which set was called
     *
     * note
     * - this may be undefined if `invalidation` was called `forKey`, instead of `forInput`
     */
    forInput: Parameters<L> | undefined;
    /**
     * the cache key that the input was serialized into
     */
    forKey: string;
    /**
     * the value which was set to the cache, defined as either
     * - the value which was set to the cache + the output from which the cached value was serialized
     * or
     * - undefined, in the case where a cache invalidation occured
     */
    value?: {
        /**
         * the direct output value returned by the logic
         */
        output: ReturnType<L>;
        /**
         * the value produced by serializing the output, with which cache.set was called
         */
        cached: CR;
    };
}) => void;
/**
 * options to configure caching for use with-simple-caching
 */
export interface WithSimpleCachingOptions<
/**
 * the response returned by the wrapped logic for the input
 */
LR extends any, 
/**
 * the response returned by the cache for the key
 */
CR extends any, 
/**
 * the logic we are caching the responses for
 */
L extends (...args: any[]) => LR> {
    cache: SimpleCache<CR> | SimpleCacheResolutionMethod<L, CR>;
    serialize?: {
        key?: KeySerializationMethod<Parameters<L>>;
        value?: (returned: LR) => CR;
    };
    deserialize?: {
        value?: (cached: CR) => LR;
    };
    secondsUntilExpiration?: number;
    hook?: {
        onSet: SimpleCacheOnSetHook<LR, CR, L>;
    };
}
/**
 * caches the promise of each invocation, based on serialization of the inputs.
 *
 * by default, this uses the `simple-in-memory-cache`, but you can pass in a `get` and `set` method for any persistent storage (e.g., local storage, cookie storage, dynamodb, etc)
 *
 * for example:
 * ```ts
 * const getApiResult = withSimpleCaching(({ name, number }) => axios.get(URL, { name, number }));
 * const result1 = getApiResult({ name: 'casey', number: 821 }); // calls the api, puts promise of results into cache, returns that promise
 * const result2 = getApiResult({ name: 'casey', number: 821 }); // returns the same promise from above, because it was found in cache - since same input as request above was used
 * expect(result1).toBe(result2); // same exact object - the promise
 * expect(await result1).toBe(await result2); // same exact object - the result of the promise
 * ```
 */
export declare const withSimpleCaching: <LR extends unknown, CR extends unknown, L extends (...args: any[]) => LR>(logic: L, { cache: cacheOption, serialize: { key: serializeKey, value: serializeValue, }, deserialize: { value: deserializeValue }, secondsUntilExpiration, hook, }: WithSimpleCachingOptions<LR, CR, L>) => L;
