"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.withSimpleCaching = exports.WithSimpleCachingOnSetTrigger = exports.getCacheFromCacheOption = exports.defaultValueSerializationMethod = exports.defaultKeySerializationMethod = exports.noOp = void 0;
const isAFunction_1 = require("./isAFunction");
const isAPromise_1 = require("./isAPromise");
const noOp = (value) => value;
exports.noOp = noOp;
const defaultKeySerializationMethod = ({ forInput }) => JSON.stringify(forInput);
exports.defaultKeySerializationMethod = defaultKeySerializationMethod;
exports.defaultValueSerializationMethod = exports.noOp;
const getCacheFromCacheOption = ({ forInput, cacheOption, }) => {
    if ((0, isAFunction_1.isAFunction)(cacheOption))
        return cacheOption({ fromInput: forInput });
    return cacheOption;
};
exports.getCacheFromCacheOption = getCacheFromCacheOption;
/**
 * enumerates all of the methods exposed by with-simple-caching which are capable of setting to the cache
 */
var WithSimpleCachingOnSetTrigger;
(function (WithSimpleCachingOnSetTrigger) {
    /**
     * the execute method sets to the cache when your wrapped logic is executed
     */
    WithSimpleCachingOnSetTrigger["EXECUTE"] = "EXECUTE";
    /**
     * the invalidate method sets to the cache when the invalidate method is executed
     */
    WithSimpleCachingOnSetTrigger["INVALIDATE"] = "INVALIDATE";
    /**
     * the update method sets to the cache when the update method is executed
     */
    WithSimpleCachingOnSetTrigger["UPDATE"] = "UPDATE";
})(WithSimpleCachingOnSetTrigger = exports.WithSimpleCachingOnSetTrigger || (exports.WithSimpleCachingOnSetTrigger = {}));
/**
 * caches the promise of each invocation, based on serialization of the inputs.
 *
 * by default, this uses the `simple-in-memory-cache`, but you can pass in a `get` and `set` method for any persistent storage (e.g., local storage, cookie storage, dynamodb, etc)
 *
 * for example:
 * ```ts
 * const getApiResult = withSimpleCaching(({ name, number }) => axios.get(URL, { name, number }));
 * const result1 = getApiResult({ name: 'casey', number: 821 }); // calls the api, puts promise of results into cache, returns that promise
 * const result2 = getApiResult({ name: 'casey', number: 821 }); // returns the same promise from above, because it was found in cache - since same input as request above was used
 * expect(result1).toBe(result2); // same exact object - the promise
 * expect(await result1).toBe(await result2); // same exact object - the result of the promise
 * ```
 */
const withSimpleCaching = (logic, { cache: cacheOption, serialize: { key: serializeKey = exports.defaultKeySerializationMethod, // default serialize key to JSON.stringify
value: serializeValue = exports.defaultValueSerializationMethod, // default serialize value to noOp
 } = {}, deserialize: { value: deserializeValue = exports.noOp } = {}, secondsUntilExpiration, hook, }) => {
    return ((...args) => {
        // define key based on args the function was invoked with
        const key = serializeKey({ forInput: args });
        // define cache based on options
        const cache = (0, exports.getCacheFromCacheOption)({ forInput: args, cacheOption });
        // start checking if its already cached
        const cachedValueOrPromise = cache.get(key);
        // define what to do with the value of this key in the cache
        const onCachedResolved = ({ cached }) => {
            // return the value if its already cached
            if (cached !== undefined)
                return deserializeValue(cached);
            // if we dont, then grab the result of the logic
            const valueOrPromise = logic(...args);
            // define the serialized value
            const serializedValue = serializeValue(valueOrPromise);
            // start setting the value into the cache
            const confirmationOrPromise = cache.set(key, serializeValue(valueOrPromise), { secondsUntilExpiration });
            // define a function for how to trigger the onSet hook, if needed
            const triggerOnSetHookIfNeeded = () => {
                if (hook === null || hook === void 0 ? void 0 : hook.onSet)
                    // note: we do not wait for the hook to resolve; hooks do not block execution ℹ️
                    hook.onSet({
                        trigger: WithSimpleCachingOnSetTrigger.EXECUTE,
                        forInput: args,
                        forKey: key,
                        value: {
                            output: valueOrPromise,
                            cached: serializedValue,
                        },
                    });
            };
            // respond to the confirmation of it being set into the cache.
            if ((0, isAPromise_1.isAPromise)(confirmationOrPromise)) {
                return confirmationOrPromise // if it was a promise
                    .then(triggerOnSetHookIfNeeded) // trigger the onset hook if needed, after the setConfirmation promise resolves
                    .then(() => valueOrPromise); // and then return the result, after setConfirmation resolves and the onSet.hook is kicked off
            }
            triggerOnSetHookIfNeeded(); // otherwise, it is already resolved, kickoff the onset hook if needed
            return valueOrPromise; // and return the value
        };
        // respond to the knowledge of whether its cached or not
        if ((0, isAPromise_1.isAPromise)(cachedValueOrPromise)) {
            return cachedValueOrPromise.then((cached) => onCachedResolved({ cached })); // if its a promise, wait for it to resolve and then run onCachedResolved
        }
        return onCachedResolved({ cached: cachedValueOrPromise }); // otherwise, its already resolved, run onCachedResolved now
    });
};
exports.withSimpleCaching = withSimpleCaching;
//# sourceMappingURL=withSimpleCaching.js.map